// Copyright (C) 2024 The Licensing Authors
// SPDX-License-Identifier: MIT

pub mod contributors {
    pub mod v1 {
        include!("gen/contributors.v1.rs");
    }
}

pub mod licenses {
    pub mod v1 {
        include!("gen/licenses.v1.rs");
    }
}

pub mod packages {
    pub mod v1 {
        include!("gen/packages.v1.rs");
    }
}

pub mod products {
    pub mod v1 {
        include!("gen/products.v1.rs");
    }
}

pub mod tokens {
    pub mod v1 {
        include!("gen/tokens.v1.rs");
    }
}

pub mod users {
    pub mod v1 {
        include!("gen/users.v1.rs");
    }
}

use tonic::codegen::StdError;
use contributors::v1::contributor_service_client::ContributorServiceClient;
use licenses::v1::license_service_client::LicenseServiceClient;
use packages::v1::package_service_client::PackageServiceClient;
use products::v1::product_service_client::ProductServiceClient;
use tokens::v1::token_service_client::TokenServiceClient;
use users::v1::user_service_client::UserServiceClient;

pub async fn new_client<D>(dst: D) -> Result<Client, Error>
where
    D: TryInto<tonic::transport::Endpoint>,
    D::Error: Into<StdError>,
{
    let d = dst.borrow();

    return Ok(Client{
        contributors: ContributorServiceClient::connect(d.clone()).await.unwrap(),
        licenses: LicenseServiceClient::connect(d.clone()).await.unwrap(),
        packages: PackageServiceClient::connect(d.clone()).await.unwrap(),
        products: ProductServiceClient::connect(d.clone()).await.unwrap(),
        tokens: TokenServiceClient::connect(d.clone()).await.unwrap(),
        users: UserServiceClient::connect(d.clone()).await.unwrap(),
    })
}

pub enum Error {

}

pub struct Client {
    contributors: ContributorServiceClient<tonic::transport::Channel>,
    licenses: LicenseServiceClient<tonic::transport::Channel>,
    packages: PackageServiceClient<tonic::transport::Channel>,
    products: ProductServiceClient<tonic::transport::Channel>,
    tokens: TokenServiceClient<tonic::transport::Channel>,
    users: UserServiceClient<tonic::transport::Channel>,
}

impl Client {
    pub fn contributors(self) -> ContributorServiceClient<tonic::transport::Channel> {
        return self.contributors
    }

    pub fn licenses(self) -> LicenseServiceClient<tonic::transport::Channel> {
        return self.licenses
    }

    pub fn packages(self) -> PackageServiceClient<tonic::transport::Channel> {
        return self.packages
    }

    pub fn products(self) -> ProductServiceClient<tonic::transport::Channel> {
        return self.products
    }

    pub fn tokens(self) -> TokenServiceClient<tonic::transport::Channel> {
        return self.tokens
    }

    pub fn users(self) -> UserServiceClient<tonic::transport::Channel> {
        return self.users
    }
}
